@@ -32,16 +32,12 @@ function create_sahara_accounts {
 
     create_service_user "sahara"
 
-    if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-        get_or_create_service "sahara" \
-                                "data-processing" \
-                                "Sahara Data Processing"
-        get_or_create_endpoint "data-processing" \
-            "$REGION_NAME" \
-            "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s" \
-            "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s" \
-            "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s"
-    fi
+    get_or_create_service "sahara" "data-processing" "Sahara Data Processing"
+    get_or_create_endpoint "data-processing" \
+        "$REGION_NAME" \
+        "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s" \
+        "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s" \
+        "$SAHARA_SERVICE_PROTOCOL://$SAHARA_SERVICE_HOST:$SAHARA_SERVICE_PORT/v1.1/\$(tenant_id)s"
 }
 
 # cleanup_sahara() - Remove residual data files, anything left over from
@@ -81,11 +77,9 @@ function configure_sahara {
     # Set configuration to send notifications
 
     if is_service_enabled ceilometer; then
-        iniset $SAHARA_CONF_FILE oslo_messaging_notifications enable "true"
         iniset $SAHARA_CONF_FILE oslo_messaging_notifications driver "messaging"
     fi
 
-    iniset $SAHARA_CONF_FILE DEFAULT verbose True
     iniset $SAHARA_CONF_FILE DEFAULT debug $ENABLE_DEBUG_LOG_LEVEL
 
     iniset $SAHARA_CONF_FILE DEFAULT plugins $SAHARA_ENABLED_PLUGINS
@@ -95,7 +89,7 @@ function configure_sahara {
 
     if is_service_enabled neutron; then
         iniset $SAHARA_CONF_FILE DEFAULT use_neutron true
-
+        iniset $SAHARA_CONF_FILE neutron endpoint_type $SAHARA_ENDPOINT_TYPE
         if is_ssl_enabled_service "neutron" \
             || is_service_enabled tls-proxy; then
             iniset $SAHARA_CONF_FILE neutron ca_file $SSL_BUNDLE_FILE
@@ -107,22 +101,32 @@ function configure_sahara {
     if is_ssl_enabled_service "heat" || is_service_enabled tls-proxy; then
         iniset $SAHARA_CONF_FILE heat ca_file $SSL_BUNDLE_FILE
     fi
+    iniset $SAHARA_CONF_FILE heat endpoint_type $SAHARA_ENDPOINT_TYPE
 
     if is_ssl_enabled_service "cinder" || is_service_enabled tls-proxy; then
         iniset $SAHARA_CONF_FILE cinder ca_file $SSL_BUNDLE_FILE
     fi
+    iniset $SAHARA_CONF_FILE cinder endpoint_type $SAHARA_ENDPOINT_TYPE
 
     if is_ssl_enabled_service "nova" || is_service_enabled tls-proxy; then
         iniset $SAHARA_CONF_FILE nova ca_file $SSL_BUNDLE_FILE
     fi
+    iniset $SAHARA_CONF_FILE nova endpoint_type $SAHARA_ENDPOINT_TYPE
 
     if is_ssl_enabled_service "swift" || is_service_enabled tls-proxy; then
         iniset $SAHARA_CONF_FILE swift ca_file $SSL_BUNDLE_FILE
     fi
+    iniset $SAHARA_CONF_FILE swift endpoint_type $SAHARA_ENDPOINT_TYPE
 
     if is_ssl_enabled_service "key" || is_service_enabled tls-proxy; then
         iniset $SAHARA_CONF_FILE keystone ca_file $SSL_BUNDLE_FILE
     fi
+    iniset $SAHARA_CONF_FILE keystone endpoint_type $SAHARA_ENDPOINT_TYPE
+
+    if is_ssl_enabled_service "glance" || is_service_enabled tls-proxy; then
+        iniset $SAHARA_CONF_FILE glance ca_file $SSL_BUNDLE_FILE
+    fi
+    iniset $SAHARA_CONF_FILE glance endpoint_type $SAHARA_ENDPOINT_TYPE
 
     # Register SSL certificates if provided
     if is_ssl_enabled_service sahara; then
@@ -148,6 +152,7 @@ function configure_sahara {
         # Enable distributed periodic tasks
         iniset $SAHARA_CONF_FILE DEFAULT periodic_coordinator_backend_url\
             $SAHARA_PERIODIC_COORDINATOR_URL
+        pip_install tooz[zookeeper]
     fi
 
     recreate_database sahara
@@ -202,8 +207,6 @@ function start_sahara {
 function configure_tempest_for_sahara {
     if is_service_enabled tempest; then
         iniset $TEMPEST_CONFIG service_available sahara True
-        # TODO(shuyingya): Delete this configuration once the Mitaka release is EOL.
-        iniset $TEMPEST_CONFIG data-processing plugin_update_support False
         iniset $TEMPEST_CONFIG data-processing-feature-enabled plugins $SAHARA_ENABLED_PLUGINS
     fi
 }
@@ -242,6 +245,7 @@ if is_service_enabled sahara; then
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         echo_summary "Initializing sahara"
         start_sahara
+    elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
         echo_summary "Configuring tempest"
         configure_tempest_for_sahara
     fi
