@@ -13,9 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import uuid
 
-import six
+from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy.orm import relationship
 
@@ -26,7 +25,7 @@ from sahara.db.sqlalchemy import types as st
 # Helpers
 
 def _generate_unicode_uuid():
-    return six.text_type(uuid.uuid4())
+    return uuidutils.generate_uuid()
 
 
 def _id_column():
@@ -72,6 +71,8 @@ class Cluster(mb.SaharaBase):
                                       cascade="all,delete",
                                       backref='cluster',
                                       lazy='joined')
+    verification = relationship('ClusterVerification', cascade="all,delete",
+                                backref="cluster", lazy='joined')
     node_groups = relationship('NodeGroup', cascade="all,delete",
                                backref='cluster', lazy='joined')
     cluster_template_id = sa.Column(sa.String(36),
@@ -81,12 +82,15 @@ class Cluster(mb.SaharaBase):
     shares = sa.Column(st.JsonListType())
     is_public = sa.Column(sa.Boolean())
     is_protected = sa.Column(sa.Boolean())
+    domain_name = sa.Column(sa.String(255))
 
     def to_dict(self, show_progress=False):
         d = super(Cluster, self).to_dict()
         d['node_groups'] = [ng.to_dict() for ng in self.node_groups]
         d['provision_progress'] = [pp.to_dict(show_progress) for pp in
                                    self.provision_progress]
+        if self.verification:
+            d['verification'] = self.verification[0].to_dict()
         return d
 
 
@@ -158,6 +162,7 @@ class Instance(mb.SaharaBase):
     management_ip = sa.Column(sa.String(45))
     volumes = sa.Column(st.JsonListType())
     storage_devices_number = sa.Column(sa.Integer)
+    dns_hostname = sa.Column(sa.String(255))
 
 
 # Template objects: ClusterTemplate, NodeGroupTemplate, TemplatesRelation
@@ -188,6 +193,7 @@ class ClusterTemplate(mb.SaharaBase):
     shares = sa.Column(st.JsonListType())
     is_public = sa.Column(sa.Boolean())
     is_protected = sa.Column(sa.Boolean())
+    domain_name = sa.Column(sa.String(255))
 
     def to_dict(self):
         d = super(ClusterTemplate, self).to_dict()
@@ -498,3 +504,54 @@ class ClusterProvisionStep(mb.SaharaBase):
         if show_progress:
             d['events'] = [event.to_dict() for event in self.events]
         return d
+
+
+class ClusterVerification(mb.SaharaBase):
+    """ClusterVerification represent results of cluster health checks."""
+
+    __tablename__ = 'cluster_verifications'
+
+    __table_args__ = (sa.UniqueConstraint('id', 'cluster_id'),)
+
+    id = _id_column()
+    cluster_id = sa.Column(
+        sa.String(36), sa.ForeignKey('clusters.id'))
+    status = sa.Column(sa.String(15))
+    checks = relationship(
+        'ClusterHealthCheck', cascade="all,delete",
+        backref='ClusterVerification', lazy='joined')
+
+    def to_dict(self):
+        base = super(ClusterVerification, self).to_dict()
+        base['checks'] = [check.to_dict() for check in self.checks]
+        return base
+
+
+class ClusterHealthCheck(mb.SaharaBase):
+    """ClusterHealthCheck respresent cluster health check."""
+
+    __tablename__ = 'cluster_health_checks'
+    __table_args__ = (sa.UniqueConstraint('id', 'verification_id'),)
+
+    id = _id_column()
+    verification_id = sa.Column(
+        sa.String(36), sa.ForeignKey('cluster_verifications.id'))
+    status = sa.Column(sa.String(15))
+    description = sa.Column(sa.Text)
+    name = sa.Column(sa.String(80))
+
+
+class PluginData(mb.SaharaBase):
+    """Plugin Data represents Provisioning Plugin."""
+
+    __tablename__ = 'plugin_data'
+    __table_args__ = (
+        sa.UniqueConstraint('name', 'tenant_id'),
+    )
+
+    id = _id_column()
+    tenant_id = sa.Column(sa.String(36), nullable=False)
+    name = sa.Column(sa.String(15), nullable=False)
+
+    plugin_labels = sa.Column(st.JsonDictType())
+    version_labels = sa.Column(st.JsonDictType())
