@@ -53,8 +53,8 @@ class SaharaException(Exception):
 class NotFoundException(SaharaException):
     code = "NOT_FOUND"
     message_template = _("Object '%s' is not found")
-    # It could be a various property of object which was not found
 
+    # It could be a various property of object which was not found
     def __init__(self, value, message_template=None):
         self.value = value
         if message_template:
@@ -65,6 +65,17 @@ class NotFoundException(SaharaException):
         super(NotFoundException, self).__init__(formatted_message)
 
 
+class NoUniqueMatchException(SaharaException):
+    code = "NO_UNIQUE_MATCH"
+    message_template = _(
+        "Response {response} is not unique for this query {query}.")
+
+    def __init__(self, response, query, message_template=None):
+        template = message_template or self.message_template
+        formatted_message = template.format(response=response, query=query)
+        super(NoUniqueMatchException, self).__init__(formatted_message)
+
+
 class NameAlreadyExistsException(SaharaException):
     code = "NAME_ALREADY_EXISTS"
     message = _("Name already exists")
@@ -225,7 +236,10 @@ class NotImplementedException(SaharaException):
     code = "NOT_IMPLEMENTED"
     message_template = _("Feature '%s' is not implemented")
 
-    def __init__(self, feature):
+    def __init__(self, feature, message_template=None):
+        if message_template:
+            self.message_template = message_template
+
         formatted_message = self.message_template % feature
 
         super(NotImplementedException, self).__init__(formatted_message)
