@@ -50,7 +50,10 @@ class ConfigHelper(object):
         'https://repository.cloudera.com/artifactory/repo/org'
         '/apache/hadoop/hadoop-openstack/2.3.0-cdh5.0.0'
         '/hadoop-openstack-2.3.0-cdh5.0.0.jar')
-    DEFAULT_EXTJS_LIB_URL = 'http://dev.sencha.com/deploy/ext-2.2.zip'
+
+    DEFAULT_EXTJS_LIB_URL = (
+        'http://tarballs.openstack.org/sahara/dist/common-artifacts/'
+        'ext-2.2.zip')
 
     SWIFT_LIB_URL = p.Config(
         'Hadoop OpenStack library URL', 'general', 'cluster', priority=1,
@@ -68,7 +71,7 @@ class ConfigHelper(object):
         'Await Cloudera agents timeout', 'general', 'cluster',
         config_type='int', priority=1, default_value=300, is_optional=True,
         description="Timeout for Cloudera agents connecting to"
-                    " Coudera Manager, in seconds")
+                    " Cloudera Manager, in seconds")
 
     AWAIT_MANAGER_STARTING_TIMEOUT = p.Config(
         'Timeout for Cloudera Manager starting', 'general', 'cluster',
