@@ -13,10 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # This file only contains utils not related to cm_api, while in
-# cloudera_utils the functios are cm_api involved.
+# cloudera_utils the functions are cm_api involved.
 
 import os
-import telnetlib
+import telnetlib  # nosec
 
 from oslo_log import log as logging
 
@@ -76,9 +76,11 @@ class CDHPluginAutoConfigsProvider(ru.HadoopAutoConfigsProvider):
 
 
 class AbstractPluginUtils(object):
-    # c_helper and db_helper will be defined in derived classes.
-    c_helper = None
-    db_helper = None
+
+    def __init__(self):
+        # c_helper and db_helper will be defined in derived classes.
+        self.c_helper = None
+        self.db_helper = None
 
     def get_role_name(self, instance, service):
         # NOTE: role name must match regexp "[_A-Za-z][-_A-Za-z0-9]{0,63}"
@@ -114,6 +116,12 @@ class AbstractPluginUtils(object):
     def get_datanodes(self, cluster):
         return u.get_instances(cluster, 'HDFS_DATANODE')
 
+    def get_hdfs_nodes(self, cluster, instances=None):
+        instances = instances if instances else u.get_instances(cluster)
+        return u.instances_with_services(
+            instances, ["HDFS_DATANODE", "HDFS_NAMENODE",
+                        "HDFS_SECONDARYNAMENODE"])
+
     def get_secondarynamenode(self, cluster):
         return u.get_instance(cluster, 'HDFS_SECONDARYNAMENODE')
 
@@ -283,15 +291,6 @@ class AbstractPluginUtils(object):
         with manager.remote() as r:
             self.db_helper.create_hive_database(cluster, r)
 
-    def create_hive_hive_directory(self, cluster):
-        # Hive requires /tmp/hive-hive directory
-        namenode = self.get_namenode(cluster)
-        with namenode.remote() as r:
-            r.execute_command(
-                'sudo su - -c "hadoop fs -mkdir -p /tmp/hive-hive" hdfs')
-            r.execute_command(
-                'sudo su - -c "hadoop fs -chown hive /tmp/hive-hive" hdfs')
-
     def install_extjs(self, cluster):
         extjs_remote_location = self.c_helper.get_extjs_lib_url(cluster)
         extjs_vm_location_dir = '/var/lib/oozie'
@@ -376,10 +375,18 @@ class AbstractPluginUtils(object):
             if config.applicable_target == service and config.name == name:
                 return types.transform_to_num(config.default_value)
         raise exc.InvalidDataException(
-            _("Unable to find config: {applicable_target: %(target)s, name: "
-              "%(name)s").format(target=service, name=name))
+            _("Unable to find config: applicable_target: {target}, name: "
+              "{name}").format(target=service, name=name))
 
     def recommend_configs(self, cluster, plugin_configs, scaling):
         provider = CDHPluginAutoConfigsProvider(
             AUTO_CONFIGURATION_SCHEMA, plugin_configs, cluster, scaling)
         provider.apply_recommended_configs()
+
+    def start_cloudera_manager(self, cluster):
+        self._start_cloudera_manager(
+            cluster, self.c_helper.AWAIT_MANAGER_STARTING_TIMEOUT)
+
+    def get_config_value(self, service, name, cluster=None):
+        configs = self.c_helper.get_plugin_configs()
+        return self._get_config_value(service, name, configs, cluster)
