@@ -1,4 +1,4 @@
-# Copyright (c) 2015 Intel Corporation
+# Copyright (c) 2016 Mirantis Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 from sahara.i18n import _
 from sahara.plugins.cdh import commands as cmd
 from sahara.plugins.cdh import deploy as common_deploy
-from sahara.plugins.cdh.v5_5_0 import cloudera_utils as cu
+from sahara.plugins.cdh.v5_9_0 import cloudera_utils as cu
 from sahara.plugins import utils as gu
 from sahara.service.edp import hdfs_helper as h
 from sahara.utils import cluster_progress_ops as cpo
@@ -62,7 +62,7 @@ PACKAGES = [
     'zookeeper'
 ]
 
-CU = cu.ClouderaUtilsV550()
+CU = cu.ClouderaUtilsV590()
 
 
 def configure_cluster(cluster):
@@ -110,12 +110,11 @@ def scale_cluster(cluster, instances):
     CU.configure_rack_awareness(cluster)
     CU.configure_instances(instances, cluster)
     CU.update_configs(instances)
-    common_deploy.prepare_scaling_kerberized_cluster(
-        cluster, CU, instances)
+    common_deploy.prepare_scaling_kerberized_cluster(cluster, CU,
+                                                     instances)
 
     CU.pu.configure_swift(cluster, instances)
     _start_roles(cluster, instances)
-
     CU.refresh_datanodes(cluster)
     CU.refresh_yarn_nodes(cluster)
     CU.restart_stale_services(cluster)
