@@ -13,31 +13,36 @@
 # under the License.
 
 
-import six
-
 import sahara.plugins.mapr.domain.node_process as np
 import sahara.plugins.mapr.domain.service as s
 import sahara.plugins.mapr.util.commands as cmd
 import sahara.plugins.mapr.util.validation_utils as vu
 
-
 DRILL = np.NodeProcess(
     name='drill-bits',
     ui_name='Drill',
     package='mapr-drill',
-    open_ports=[]
+    open_ports=[8047]
+)
+
+DRILL_YARN = np.NodeProcess(
+    name='drill-bits-yarn',
+    ui_name='Drill on YARN',
+    package='mapr-drill-yarn',
+    open_ports=[8047]
 )
 
 
-@six.add_metaclass(s.Single)
 class Drill(s.Service):
     def __init__(self):
         super(Drill, self).__init__()
         self._name = 'drill'
         self._ui_name = 'Drill'
         self._node_processes = [DRILL]
-        self._ui_info = [('Drill', DRILL, 'http://%s:8047')]
-        self._validation_rules = [vu.at_least(1, DRILL)]
+        self._ui_info = [('Drill', DRILL, {s.SERVICE_UI: 'http://%s:8047'})]
+        self._validation_rules = [
+            vu.node_client_package_conflict_vr([DRILL], DRILL_YARN)
+        ]
 
     def install(self, cluster_context, instances):
         # Drill requires running cluster
@@ -46,34 +51,62 @@ class Drill(s.Service):
     def post_start(self, cluster_context, instances):
         instances = instances or cluster_context.get_instances(DRILL)
         super(Drill, self).install(cluster_context, instances)
+        self._set_service_dir_owner(cluster_context, instances)
         for instance in instances:
-            cmd.chown(instance, 'mapr:mapr', self.service_dir(cluster_context))
             cmd.re_configure_sh(instance, cluster_context)
 
 
-@six.add_metaclass(s.Single)
 class DrillV07(Drill):
     def __init__(self):
         super(DrillV07, self).__init__()
         self._version = '0.7'
 
 
-@six.add_metaclass(s.Single)
 class DrillV08(Drill):
     def __init__(self):
         super(DrillV08, self).__init__()
         self._version = '0.8'
 
 
-@six.add_metaclass(s.Single)
 class DrillV09(Drill):
     def __init__(self):
         super(DrillV09, self).__init__()
         self._version = '0.9'
 
 
-@six.add_metaclass(s.Single)
 class DrillV11(Drill):
     def __init__(self):
         super(DrillV11, self).__init__()
         self._version = "1.1"
+
+
+class DrillV12(Drill):
+    def __init__(self):
+        super(DrillV12, self).__init__()
+        self._version = "1.2"
+
+
+class DrillV14(Drill):
+    def __init__(self):
+        super(DrillV14, self).__init__()
+        self._version = "1.4"
+
+
+class DrillV16(Drill):
+    def __init__(self):
+        super(DrillV16, self).__init__()
+        self._version = "1.6"
+
+
+class DrillV18(Drill):
+    def __init__(self):
+        super(DrillV18, self).__init__()
+        self._version = "1.8"
+        self._node_processes = [DRILL, DRILL_YARN]
+
+
+class DrillV19(Drill):
+    def __init__(self):
+        super(DrillV19, self).__init__()
+        self._version = "1.8"
+        self._node_processes = [DRILL, DRILL_YARN]
