@@ -14,15 +14,16 @@
 
 from oslo_log import log as logging
 
-import sahara.context as con
+import sahara.context as context
+from sahara.i18n import _
 import sahara.plugins.mapr.domain.configuration_file as bcf
 import sahara.plugins.mapr.domain.node_process as np
 import sahara.plugins.mapr.domain.service as s
 import sahara.plugins.mapr.services.mysql.mysql as mysql
+import sahara.plugins.mapr.util.event_log as el
 import sahara.plugins.mapr.util.general as g
 import sahara.plugins.mapr.util.validation_utils as vu
 
-
 LOG = logging.getLogger(__name__)
 OOZIE_START_DELAY = 30
 
@@ -42,7 +43,8 @@ class Oozie(s.Service):
         self._node_processes = [OOZIE]
         self._cluster_defaults = ['oozie-default.json']
         self._validation_rules = [vu.exactly(1, OOZIE)]
-        self._ui_info = [('Oozie', OOZIE, 'http://%s:11000/oozie')]
+        self._ui_info = [('Oozie', OOZIE,
+                          {s.SERVICE_UI: 'http://%s:11000/oozie'})]
 
     def libext_path(self):
         return '/opt/mapr/oozie/oozie-%s/oozie-server/lib/' % self.version
@@ -56,34 +58,32 @@ class Oozie(s.Service):
         oozie_site.add_properties(self._get_oozie_site_props(cluster_context))
         return [oozie_site]
 
-    def _get_oozie_site_props(self, context):
+    def _get_oozie_site_props(self, cluster_context):
         oozie_specs = mysql.MySQL.OOZIE_SPECS
 
         return {
             'oozie.db.schema.name': oozie_specs.db_name,
             'oozie.service.JPAService.create.db.schema': True,
             'oozie.service.JPAService.jdbc.driver': mysql.MySQL.DRIVER_CLASS,
-            'oozie.service.JPAService.jdbc.url': self._get_jdbc_uri(context),
+            'oozie.service.JPAService.jdbc.url': self._get_jdbc_uri(
+                cluster_context),
             'oozie.service.JPAService.jdbc.username': oozie_specs.user,
             'oozie.service.JPAService.jdbc.password': oozie_specs.password,
             'oozie.service.HadoopAccessorService.hadoop.configurations':
-                '*=%s' % context.hadoop_conf
+                '*=%s' % cluster_context.hadoop_conf
         }
 
-    def _get_jdbc_uri(self, context):
+    def _get_jdbc_uri(self, cluster_context):
         jdbc_uri = ('jdbc:mysql://%(db_host)s:%(db_port)s/%(db_name)s?'
                     'createDatabaseIfNotExist=true')
         jdbc_args = {
-            'db_host': mysql.MySQL.get_db_instance(context).internal_ip,
+            'db_host': mysql.MySQL.get_db_instance(
+                cluster_context).internal_ip,
             'db_port': mysql.MySQL.MYSQL_SERVER_PORT,
             'db_name': mysql.MySQL.OOZIE_SPECS.db_name,
         }
         return jdbc_uri % jdbc_args
 
-    def _set_owner(self, remote):
-        remote.execute_command('chown -R mapr:mapr /opt/mapr/oozie',
-                               run_as_root=True)
-
     def install(self, cluster_context, instances):
         # oozie requires executed configure.sh
         pass
@@ -92,7 +92,6 @@ class Oozie(s.Service):
         super(Oozie, self).install(cluster_context, instances)
         oozie_instances = cluster_context.filter_instances(instances,
                                                            service=self)
-
         for instance in oozie_instances:
             with instance.remote() as r:
                 r.execute_command(
@@ -106,18 +105,19 @@ class Oozie(s.Service):
 
         if oozie_service:
             symlink_cmd = (
-                'cp /usr/share/java/mysql-connector-java.jar %s' %
+                'cp /opt/mapr/lib/mysql-connector-*.jar %s' %
                 self.libext_path())
             with oozie_inst.remote() as r:
                 LOG.debug('Installing MySQL connector for Oozie')
                 r.execute_command(symlink_cmd, run_as_root=True,
                                   raise_when_error=False)
-                self._set_owner(r)
+        self._set_service_dir_owner(cluster_context, instances)
 
     def post_start(self, cluster_context, instances):
         instances = cluster_context.filter_instances(instances, OOZIE)
         self._rebuild(cluster_context, instances)
 
+    @el.provision_event(instance_reference=1)
     @g.remote_command(1)
     def _rebuild_oozie_war(self, remote, cluster_context):
         cmd = '%(home)s/bin/oozie-setup.sh -hadoop %(version)s' \
@@ -131,12 +131,13 @@ class Oozie(s.Service):
         instances = cluster_context.filter_instances(instances, OOZIE)
         self._rebuild(cluster_context, instances)
 
+    @el.provision_step(_("Rebuilt Oozie war"))
     def _rebuild(self, cluster_context, instances):
         OOZIE.stop(filter(OOZIE.is_started, instances))
         g.execute_on_instances(
             instances, self._rebuild_oozie_war, cluster_context)
         OOZIE.start(instances)
-        con.sleep(OOZIE_START_DELAY)
+        context.sleep(OOZIE_START_DELAY)
 
 
 class OozieV401(Oozie):
