@@ -26,11 +26,12 @@ class Context(bc.BaseClusterContext):
         self._cluster_mode = yarn.YARNv270.cluster_mode
         self._node_aware = True
         self._resource_manager_uri = "maprfs:///"
-        self._mapr_version = "5.0.0"
+        self._mapr_version = "5.2.0"
         self._ubuntu_ecosystem_repo = (
-            "http://package.mapr.com/releases/ecosystem-5.x/ubuntu binary/")
+            "http://package.mapr.com/releases/MEP/MEP-2.0.0/ubuntu/"
+            " binary trusty")
         self._centos_ecosystem_repo = (
-            "http://package.mapr.com/releases/ecosystem-5.x/redhat")
+            "http://package.mapr.com/releases/MEP/MEP-2.0.0/redhat")
 
     @property
     def hadoop_lib(self):
