@@ -44,8 +44,7 @@ XML_CONFS = {
 }
 
 _default_executor_classpath = ":".join(
-    ['/usr/lib/hadoop/lib/jackson-core-asl-1.8.8.jar',
-     '/usr/lib/hadoop/hadoop-swift.jar'])
+    ['/usr/lib/hadoop-mapreduce/hadoop-openstack.jar'])
 
 SPARK_CONFS = {
     'Spark': {
@@ -156,6 +155,11 @@ ENABLE_SWIFT = p.Config('Enable Swift', 'general', 'cluster',
                         config_type="bool", priority=1,
                         default_value=True, is_optional=False)
 
+DATANODES_STARTUP_TIMEOUT = p.Config(
+    'DataNodes startup timeout', 'general', 'cluster', config_type='int',
+    priority=1, default_value=10800, is_optional=True,
+    description='Timeout for DataNodes startup, in seconds')
+
 # Default set to 1 day, which is the default Keystone token
 # expiration time. After the token is expired we can't continue
 # scaling anyway.
@@ -223,6 +227,7 @@ def _initialise_configs():
 
     configs.append(DECOMMISSIONING_TIMEOUT)
     configs.append(ENABLE_SWIFT)
+    configs.append(DATANODES_STARTUP_TIMEOUT)
     if CONF.enable_data_locality:
         configs.append(ENABLE_DATA_LOCALITY)
 
@@ -366,7 +371,7 @@ def generate_spark_env_configs(cluster):
     return '\n'.join(configs)
 
 
-# workernames need to be a list of woker names
+# workernames need to be a list of worker names
 def generate_spark_slaves_configs(workernames):
     return '\n'.join(workernames)
 
