@@ -13,13 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import jsonutils as json
 import six
 import yaml
 
 from sahara.plugins import provisioning as plugin_provisioning
+from sahara.service.heat import commons as heat_common
+from sahara.utils import cluster as cl
 from sahara.utils import general as g
 from sahara.utils.openstack import base as b
 from sahara.utils.openstack import heat as h
@@ -30,6 +34,32 @@ LOG = logging.getLogger(__name__)
 
 SSH_PORT = 22
 INSTANCE_RESOURCE_NAME = "inst"
+SERVER_GROUP_PARAM_NAME = "servgroup"
+AUTO_SECURITY_GROUP_PARAM_NAME = "autosecgroup"
+INTERNAL_DESIGNATE_REC = "internal_designate_record"
+INTERNAL_DESIGNATE_REV_REC = "internal_designate_reverse_record"
+EXTERNAL_DESIGNATE_REC = "external_designate_record"
+EXTERNAL_DESIGNATE_REV_REC = "external_designate_reverse_record"
+
+# TODO(vgridnev): Using insecure flag until correct way to pass certificate
+# will be invented
+WAIT_CONDITION_SCRIPT_TEMPLATE = '''
+while true; do
+    wc_notify --insecure --data-binary '{"status": "SUCCESS"}'
+    if [ $? -eq 0 ]; then
+        break
+    fi
+    sleep 10
+done
+'''
+
+heat_engine_opts = [
+    cfg.BoolOpt(
+        'heat_enable_wait_condition', default=True,
+        help="Enable wait condition feature to reduce polling during cluster "
+             "creation")
+]
+CONF.register_opts(heat_engine_opts)
 
 
 def _get_inst_name(ng):
@@ -42,6 +72,14 @@ def _get_inst_name(ng):
     }
 
 
+def _get_inst_domain_name(domain):
+    return {
+        "list_join": [
+            '.',
+            [{"get_attr": [INSTANCE_RESOURCE_NAME, "name"]}, domain]]
+    }
+
+
 def _get_aa_group_name(cluster):
     return g.generate_aa_group_name(cluster.name)
 
@@ -102,9 +140,28 @@ class ClusterStack(object):
     def __init__(self, cluster):
         self.cluster = cluster
         self.node_groups_extra = {}
-        self.heat_stack = None
         self.files = {}
         self.last_updated_time = None
+        self.base_info = (
+            "Data Processing Cluster by Sahara\n"
+            "Sahara cluster name: {cluster}\n"
+            "Sahara engine: {version}".format(
+                cluster=cluster.name, version=heat_common.HEAT_ENGINE_VERSION)
+        )
+
+    def _node_group_description(self, ng):
+        return "{info}\nNode group {node_group}".format(
+            info=self.base_info, node_group=ng.name)
+
+    def _asg_for_node_group_description(self, ng):
+        return ("{info}\n"
+                "Auto security group for Sahara Node Group: "
+                "{node_group}".format(info=self.base_info, node_group=ng.name))
+
+    def _volume_for_node_group_description(self, ng):
+        return ("{info}\n"
+                "Volume for Sahara Node Group {node_group}".format(
+                    node_group=ng.name, info=self.base_info))
 
     def add_node_group_extra(self, node_group_id, node_count,
                              gen_userdata_func):
@@ -117,41 +174,44 @@ class ClusterStack(object):
         outputs = {}
         resources = self._serialize_resources(outputs)
         return yaml.safe_dump({
-            "heat_template_version": "2013-05-23",
-            "description": "Data Processing Cluster by Sahara",
+            "heat_template_version": heat_common.HEAT_TEMPLATE_VERSION,
+            "description": self.base_info,
             "resources": resources,
             "outputs": outputs
         })
 
     def instantiate(self, update_existing, disable_rollback=True):
         main_tmpl = self._get_main_template()
-
-        heat = h.client()
-
         kwargs = {
             'stack_name': self.cluster.stack_name,
             'timeout_mins': 180,
             'disable_rollback': disable_rollback,
             'parameters': {},
             'template': main_tmpl,
-            'files': self.files}
+            'files': self.files
+        }
 
         if CONF.heat_stack_tags:
             kwargs['tags'] = ",".join(CONF.heat_stack_tags)
 
+        log_kwargs = copy.deepcopy(kwargs)
+        log_kwargs['template'] = yaml.safe_load(log_kwargs['template'])
+        for filename in log_kwargs['files'].keys():
+            log_kwargs['files'][filename] = yaml.safe_load(
+                log_kwargs['files'][filename])
+        log_kwargs = json.dumps(log_kwargs)
+
         if not update_existing:
-            LOG.debug("Creating Heat stack with args: {args}"
-                      .format(args=kwargs))
-            b.execute_with_retries(heat.stacks.create, **kwargs)
+            LOG.debug("Creating Heat stack with args: \n{args}"
+                      .format(args=log_kwargs))
+            b.execute_with_retries(h.client().stacks.create, **kwargs)
         else:
             stack = h.get_stack(self.cluster.stack_name)
             self.last_updated_time = stack.updated_time
-            LOG.debug("Updating Heat stack {stack} with args: "
-                      "{args}".format(stack=stack, args=kwargs))
+            LOG.debug("Updating Heat stack {stack} with args: \n"
+                      "{args}".format(stack=stack, args=log_kwargs))
             b.execute_with_retries(stack.update, **kwargs)
 
-        self.heat_stack = h.get_stack(self.cluster.stack_name)
-
     def _need_aa_server_group(self, node_group):
         for node_process in node_group.node_processes:
             if node_process in self.cluster.anti_affinity:
@@ -165,8 +225,7 @@ class ClusterStack(object):
         return {
             "scheduler_hints": {
                 "group": {
-                    "get_resource": _get_aa_group_name(
-                        self.cluster)
+                    "get_param": SERVER_GROUP_PARAM_NAME,
                 }
             }
         }
@@ -180,6 +239,9 @@ class ClusterStack(object):
         for ng in self.cluster.node_groups:
             resources.update(self._serialize_ng_group(ng, outputs))
 
+        for ng in self.cluster.node_groups:
+            resources.update(self._serialize_auto_security_group(ng))
+
         return resources
 
     def _serialize_ng_group(self, ng, outputs):
@@ -188,6 +250,15 @@ class ClusterStack(object):
 
         outputs[ng.name + "-instances"] = {
             "value": {"get_attr": [ng.name, "instance"]}}
+        properties = {"instance_index": "%index%"}
+
+        if ng.cluster.anti_affinity:
+            properties[SERVER_GROUP_PARAM_NAME] = {
+                'get_resource': _get_aa_group_name(ng.cluster)}
+
+        if ng.auto_security_group:
+            properties[AUTO_SECURITY_GROUP_PARAM_NAME] = {
+                'get_resource': g.generate_auto_security_group_name(ng)}
 
         return {
             ng.name: {
@@ -196,22 +267,25 @@ class ClusterStack(object):
                     "count": self.node_groups_extra[ng.id]['node_count'],
                     "resource_def": {
                         "type": ng_file_name,
-                        "properties": {"instance_index": "%index%"}
+                        "properties": properties
                     }
                 }
             }
         }
 
     def _serialize_ng_file(self, ng):
+        parameters = {"instance_index": {"type": "string"}}
+
+        if ng.cluster.anti_affinity:
+            parameters[SERVER_GROUP_PARAM_NAME] = {'type': "string"}
+
+        if ng.auto_security_group:
+            parameters[AUTO_SECURITY_GROUP_PARAM_NAME] = {'type': "string"}
+
         return yaml.safe_dump({
-            "heat_template_version": "2013-05-23",
-            "description": "Node Group {node_group} of "
-                           "cluster {cluster}".format(node_group=ng.name,
-                                                      cluster=ng.cluster.name),
-            "parameters": {
-                "instance_index": {
-                    "type": "string"
-                }},
+            "heat_template_version": heat_common.HEAT_TEMPLATE_VERSION,
+            "description": self._node_group_description(ng),
+            "parameters": parameters,
             "resources": self._serialize_instance(ng),
             "outputs": {
                 "instance": {"value": {
@@ -221,10 +295,10 @@ class ClusterStack(object):
         })
 
     def _serialize_auto_security_group(self, ng):
+        if not ng.auto_security_group:
+            return {}
         security_group_name = g.generate_auto_security_group_name(ng)
-        security_group_description = (
-            "Auto security group created by Sahara for Node Group "
-            "'%s' of cluster '%s'." % (ng.name, ng.cluster.name))
+        security_group_description = self._asg_for_node_group_description(ng)
 
         if CONF.use_neutron:
             res_type = "OS::Neutron::SecurityGroup"
@@ -286,31 +360,90 @@ class ClusterStack(object):
         return int(configs.get(cfg_target,
                                {}).get(cfg_name, timeout_cfg.default_value))
 
+    def _serialize_designate_records(self):
+        if not self.cluster.use_designate_feature():
+            return {}
+        hostname = _get_inst_domain_name(self.cluster.domain_name)
+        return {
+            INTERNAL_DESIGNATE_REC: {
+                'type': 'OS::Designate::Record',
+                'properties': {
+                    'name': hostname,
+                    'type': 'A',
+                    'data': {'get_attr': [
+                        INSTANCE_RESOURCE_NAME, 'networks', 'private', 0]},
+                    'domain': self.cluster.domain_name
+                }
+            },
+            EXTERNAL_DESIGNATE_REC: {
+                'type': 'OS::Designate::Record',
+                'properties': {
+                    'name': hostname,
+                    'type': 'A',
+                    'data': {'get_attr': ['floating_ip', 'ip']},
+                    'domain': self.cluster.domain_name
+                }
+            }
+        }
+
+    def _serialize_designate_reverse_records(self):
+
+        if not self.cluster.use_designate_feature():
+            return {}
+
+        def _generate_reversed_ip(ip):
+            return {
+                'list_join': [
+                    '.',
+                    [
+                        {'str_split': ['.', ip, 3]},
+                        {'str_split': ['.', ip, 2]},
+                        {'str_split': ['.', ip, 1]},
+                        {'str_split': ['.', ip, 0]},
+                        'in-addr.arpa.'
+                    ]
+                ]
+            }
+
+        hostname = _get_inst_domain_name(self.cluster.domain_name)
+        return {
+            INTERNAL_DESIGNATE_REV_REC: {
+                'type': 'OS::Designate::Record',
+                'properties': {
+                    'name': _generate_reversed_ip({'get_attr': [
+                        INSTANCE_RESOURCE_NAME, 'networks', 'private', 0]}),
+                    'type': 'PTR',
+                    'data': hostname,
+                    'domain': 'in-addr.arpa.'
+                }
+            },
+            EXTERNAL_DESIGNATE_REV_REC: {
+                'type': 'OS::Designate::Record',
+                'properties': {
+                    'name':  _generate_reversed_ip(
+                        {'get_attr': ['floating_ip', 'ip']}),
+                    'type': 'PTR',
+                    'data': hostname,
+                    'domain': 'in-addr.arpa.'
+                }
+            }
+        }
+
     def _serialize_instance(self, ng):
         resources = {}
         properties = {}
 
         inst_name = _get_inst_name(ng)
 
-        if ng.auto_security_group:
-            resources.update(self._serialize_auto_security_group(ng))
+        if ng.floating_ip_pool:
+            resources.update(self._serialize_nova_floating(ng))
 
         if CONF.use_neutron:
-            port_name = _get_port_name(ng)
-            resources.update(self._serialize_port(
-                port_name, self.cluster.neutron_management_network,
-                self._get_security_groups(ng)))
+            properties["networks"] = [{
+                "network": self.cluster.neutron_management_network}]
 
-            properties["networks"] = [{"port": {"get_resource": "port"}}]
-
-            if ng.floating_ip_pool:
-                resources.update(self._serialize_neutron_floating(ng))
-        else:
-            if ng.floating_ip_pool:
-                resources.update(self._serialize_nova_floating(ng))
-
-            if ng.security_groups or ng.auto_security_group:
-                properties["security_groups"] = self._get_security_groups(ng)
+        if ng.security_groups or ng.auto_security_group:
+            properties["security_groups"] = self._get_security_groups(ng)
 
         # Check if cluster contains user key-pair and include it to template.
         if self.cluster.user_keypair_id:
@@ -318,21 +451,28 @@ class ClusterStack(object):
 
         gen_userdata_func = self.node_groups_extra[ng.id]['gen_userdata_func']
         key_script = gen_userdata_func(ng, inst_name)
-        wait_condition_script = (
-            "wc_notify --data-binary '{\"status\": \"SUCCESS\"}'")
-        userdata = {
-            "str_replace": {
-                "template": "\n".join([key_script, wait_condition_script]),
-                "params": {
-                    "wc_notify": {
-                        "get_attr": [
-                            _get_wc_handle_name(ng.name),
-                            "curl_cli"
-                        ]
+        if CONF.heat_enable_wait_condition:
+            etc_hosts = cl.etc_hosts_entry_for_service('orchestration')
+            if etc_hosts:
+                etc_hosts = "echo '%s' | sudo tee -a /etc/hosts" % etc_hosts
+            tml = [key_script, WAIT_CONDITION_SCRIPT_TEMPLATE]
+            if etc_hosts:
+                tml = [key_script, etc_hosts, WAIT_CONDITION_SCRIPT_TEMPLATE]
+            userdata = {
+                "str_replace": {
+                    "template": "\n".join(tml),
+                    "params": {
+                        "wc_notify": {
+                            "get_attr": [
+                                _get_wc_handle_name(ng.name),
+                                "curl_cli"
+                            ]
+                        }
                     }
                 }
             }
-        }
+        else:
+            userdata = key_script
 
         if ng.availability_zone:
             properties["availability_zone"] = ng.availability_zone
@@ -354,11 +494,16 @@ class ClusterStack(object):
             }
         })
 
-        if ng.volumes_per_node > 0 and ng.volumes_size > 0:
-            resources.update(self._serialize_volume(ng))
-
+        resources.update(self._serialize_designate_records())
+        resources.update(self._serialize_designate_reverse_records())
         resources.update(self._serialize_volume(ng))
-        resources.update({
+        resources.update(self._serialize_wait_condition(ng))
+        return resources
+
+    def _serialize_wait_condition(self, ng):
+        if not CONF.heat_enable_wait_condition:
+            return {}
+        return {
             _get_wc_handle_name(ng.name): {
                 "type": "OS::Heat::WaitConditionHandle"
             },
@@ -370,34 +515,6 @@ class ClusterStack(object):
                     "handle": {"get_resource": _get_wc_handle_name(ng.name)}
                 }
             }
-        })
-        return resources
-
-    def _serialize_port(self, port_name, fixed_net_id, security_groups):
-        properties = {
-            "network_id": fixed_net_id,
-            "replacement_policy": "AUTO",
-            "name": port_name
-        }
-        if security_groups:
-            properties["security_groups"] = security_groups
-
-        return {
-            "port": {
-                "type": "OS::Neutron::Port",
-                "properties": properties,
-            }
-        }
-
-    def _serialize_neutron_floating(self, ng):
-        return {
-            "floating_ip": {
-                "type": "OS::Neutron::FloatingIP",
-                "properties": {
-                    "floating_network_id": ng.floating_ip_pool,
-                    "port_id": {"get_resource": "port"}
-                }
-            }
         }
 
     def _serialize_nova_floating(self, ng):
@@ -418,6 +535,8 @@ class ClusterStack(object):
         }
 
     def _serialize_volume(self, ng):
+        if not ng.volumes_size or not ng.volumes_per_node:
+            return {}
         volume_file_name = "file://" + ng.name + "-volume.yaml"
         self.files[volume_file_name] = self._serialize_volume_file(ng)
 
@@ -455,10 +574,8 @@ class ClusterStack(object):
                 "local_to_instance": {"get_param": "instance"}}
 
         return yaml.safe_dump({
-            "heat_template_version": "2013-05-23",
-            "description": "Volume for node Group {node_group} of "
-                           "cluster {cluster}".format(node_group=ng.name,
-                                                      cluster=ng.cluster.name),
+            "heat_template_version": heat_common.HEAT_TEMPLATE_VERSION,
+            "description": self._volume_for_node_group_description(ng),
             "parameters": {
                 "volume_index": {
                     "type": "string"
@@ -485,12 +602,11 @@ class ClusterStack(object):
         })
 
     def _get_security_groups(self, node_group):
-        if not node_group.auto_security_group:
-            return node_group.security_groups
         node_group_sg = list(node_group.security_groups or [])
-        node_group_sg += [
-            {"get_resource": g.generate_auto_security_group_name(node_group)}
-        ]
+        if node_group.auto_security_group:
+            node_group_sg += [
+                {"get_param": AUTO_SECURITY_GROUP_PARAM_NAME}
+            ]
         return node_group_sg
 
     def _serialize_aa_server_group(self):
@@ -506,7 +622,9 @@ class ClusterStack(object):
         }
 
     def get_node_group_instances(self, node_group):
-        for output in self.heat_stack.outputs:
+        cluster = node_group.cluster
+        outputs = h.get_stack_outputs(cluster)
+        for output in outputs:
             if output['output_key'] == node_group.name + "-instances":
                 return output["output_value"]
 
