@@ -16,17 +16,16 @@
 import mock
 import six
 
-from sahara.plugins.cdh.v5_3_0.edp_engine import EdpOozieEngine
-from sahara.plugins.cdh.v5_3_0.edp_engine import EdpSparkEngine
-from sahara.plugins.cdh.v5_3_0 import versionhandler
+from sahara.plugins.cdh.v5_4_0 import edp_engine
+from sahara.plugins.cdh.v5_4_0 import versionhandler
 from sahara.tests.unit import base
 
 
 class VersionHandlerTestCase(base.SaharaTestCase):
 
-    plugin_path = "sahara.plugins.cdh.v5_3_0."
-    cloudera_utils_path = plugin_path + "cloudera_utils.ClouderaUtilsV530."
-    plugin_utils_path = plugin_path + "plugin_utils.PluginUtilsV530."
+    plugin_path = "sahara.plugins.cdh.v5_4_0."
+    cloudera_utils_path = plugin_path + "cloudera_utils.ClouderaUtilsV540."
+    plugin_utils_path = plugin_path + "plugin_utils.PluginUtilsV540."
 
     def setUp(self):
         super(VersionHandlerTestCase, self).setUp()
@@ -61,7 +60,7 @@ class VersionHandlerTestCase(base.SaharaTestCase):
         self.vh._set_cluster_info.assert_called_once_with(cluster)
 
     @mock.patch(plugin_path + "deploy.decommission_cluster")
-    def test_decommmission_nodes(self, decommission_cluster):
+    def test_decommission_nodes(self, decommission_cluster):
         cluster = mock.Mock()
         instances = mock.Mock()
         self.vh.decommission_nodes(cluster, instances)
@@ -82,7 +81,9 @@ class VersionHandlerTestCase(base.SaharaTestCase):
     @mock.patch(plugin_utils_path + "get_hue")
     def test_set_cluster_info(self, get_hue, get_cloudera_manager_info,
                               ctx, cluster_update):
-        get_hue.return_value.management_ip = "1.2.3.4"
+        hue = mock.Mock()
+        hue.get_ip_or_dns_name.return_value = "1.2.3.4"
+        get_hue.return_value = hue
         cluster = mock.Mock()
         self.vh._set_cluster_info(cluster)
         info = {'info': {'Hue Dashboard': {'Web UI': 'http://1.2.3.4:8888'}}}
@@ -96,11 +97,11 @@ class VersionHandlerTestCase(base.SaharaTestCase):
         cluster = mock.Mock()
         job_type = 'Java'
         ret = self.vh.get_edp_engine(cluster, job_type)
-        self.assertIsInstance(ret, EdpOozieEngine)
+        self.assertIsInstance(ret, edp_engine.EdpOozieEngine)
 
         job_type = 'Spark'
         ret = self.vh.get_edp_engine(cluster, job_type)
-        self.assertIsInstance(ret, EdpSparkEngine)
+        self.assertIsInstance(ret, edp_engine.EdpSparkEngine)
 
         job_type = 'unsupported'
         ret = self.vh.get_edp_engine(cluster, job_type)
@@ -108,8 +109,8 @@ class VersionHandlerTestCase(base.SaharaTestCase):
 
     def test_get_edp_job_types(self):
         ret = self.vh.get_edp_job_types()
-        expect = EdpOozieEngine.get_supported_job_types() + \
-            EdpSparkEngine.get_supported_job_types()
+        expect = edp_engine.EdpOozieEngine.get_supported_job_types() + \
+            edp_engine.EdpSparkEngine.get_supported_job_types()
         self.assertEqual(expect, ret)
 
     @mock.patch(plugin_path +
