@@ -18,7 +18,7 @@ import six
 import testtools
 
 from sahara import exceptions
-from sahara.service import api
+from sahara.service.api import v10 as api
 from sahara.service.validations import clusters as c
 from sahara.service.validations import clusters_schema as c_schema
 from sahara.tests.unit import base
@@ -158,6 +158,7 @@ class TestClusterCreateValidation(u.ValidationTestCase):
         )
 
     def test_cluster_create_mixed_nova_neutron(self):
+        self.override_config("use_neutron", False)
         self._assert_create_object_validation(
             data={
                 'name': "test-name",
@@ -289,10 +290,7 @@ class TestClusterCreateValidation(u.ValidationTestCase):
                         'security_groups': ['group1', 'group2']
                     }
                 ]
-            },
-            bad_req_i=(1, 'MISSING_FLOATING_NETWORK',
-                       "Node Group ng2 is missing 'floating_ip_pool' "
-                       "field")
+            }
         )
 
     def test_cluster_create_with_proxy_gateway(self):
@@ -381,7 +379,6 @@ class TestClusterCreateValidation(u.ValidationTestCase):
         )
 
     def test_cluster_create_availability_zone(self):
-        self.override_config('use_neutron', True)
         self._assert_create_object_validation(
             data={
                 'name': 'testname',
@@ -410,7 +407,6 @@ class TestClusterCreateValidation(u.ValidationTestCase):
         )
 
     def test_cluster_create_wrong_availability_zone(self):
-        self.override_config('use_neutron', True)
         self._assert_create_object_validation(
             data={
                 'name': 'testname',
@@ -438,7 +434,6 @@ class TestClusterCreateValidation(u.ValidationTestCase):
         )
 
     def test_cluster_create_wrong_volumes_availability_zone(self):
-        self.override_config('use_neutron', True)
         self._assert_create_object_validation(
             data={
                 'name': 'testname',
@@ -486,7 +481,6 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "sahara.service.validations.base.check_plugin_supports_version",
             "sahara.service.validations.base._get_plugin_configs",
             "sahara.service.validations.base.check_node_processes",
-            "sahara.utils.openstack.nova.client",
         ]
         self.patchers = []
         for module in modules:
@@ -536,6 +530,8 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "plugin_name": "fake",
             "hadoop_version": "0.1",
             "cluster_template_id": '%s' % ctmpl_id,
+            "neutron_management_network": "d9a3bebc-f788-4b81-"
+                                          "9a93-aa048022c1ca",
             'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
         }
         patchers = u.start_patch(False)
@@ -546,6 +542,8 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "name": "testwithnodegroups",
             "plugin_name": "fake",
             "hadoop_version": "0.1",
+            "neutron_management_network": "d9a3bebc-f788-4b81-"
+                                          "9a93-aa048022c1ca",
             "node_groups": [
                 {
                     "name": "allinone",
@@ -580,6 +578,8 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "name": "testwithnodegroups",
             "plugin_name": "fake",
             "hadoop_version": "0.1",
+            "neutron_management_network": "d9a3bebc-f788-4b81-"
+                                          "9a93-aa048022c1ca",
             "node_groups": [
                 {
                     "name": "allinone",
@@ -618,6 +618,8 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "plugin_name": "fake",
             "hadoop_version": "0.1",
             "cluster_template_id": '%s' % ctmpl_id,
+            "neutron_management_network": "d9a3bebc-f788-4b81-"
+                                          "9a93-aa048022c1ca",
             'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
             "node_groups": [
                 {
@@ -643,6 +645,8 @@ class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
             "name": "testtmplnodegroups",
             "plugin_name": "fake",
             "hadoop_version": "0.1",
+            "neutron_management_network": "d9a3bebc-f788-4b81-"
+                                          "9a93-aa048022c1ca",
             "node_groups": [
                 {
                     "node_group_template_id": '%s' % ng_id,
