@@ -121,24 +121,26 @@ def _get_security_groups_list():
 
 
 def start_patch(patch_templates=True):
-    get_clusters_p = mock.patch("sahara.service.api.get_clusters")
-    get_cluster_p = mock.patch("sahara.service.api.get_cluster")
+    get_clusters_p = mock.patch("sahara.service.api.v10.get_clusters")
+    get_cluster_p = mock.patch("sahara.service.api.v10.get_cluster")
     if patch_templates:
         get_ng_templates_p = mock.patch(
-            "sahara.service.api.get_node_group_templates")
+            "sahara.service.api.v10.get_node_group_templates")
         get_ng_template_p = mock.patch(
-            "sahara.service.api.get_node_group_template")
+            "sahara.service.api.v10.get_node_group_template")
     if patch_templates:
         get_cl_templates_p = mock.patch(
-            "sahara.service.api.get_cluster_templates")
+            "sahara.service.api.v10.get_cluster_templates")
         get_cl_template_p = mock.patch(
-            "sahara.service.api.get_cluster_template")
+            "sahara.service.api.v10.get_cluster_template")
     nova_p = mock.patch("sahara.utils.openstack.nova.client")
     heat_p = mock.patch("sahara.utils.openstack.heat.client")
+    image_manager_p = mock.patch(
+        "sahara.utils.openstack.images.SaharaImageManager")
     cinder_p = mock.patch("sahara.utils.openstack.cinder.client")
     cinder_exists_p = mock.patch(
         "sahara.utils.openstack.cinder.check_cinder_exists")
-    get_image_p = mock.patch("sahara.service.api.get_image")
+    get_image_p = mock.patch("sahara.service.api.v10.get_image")
 
     get_image = get_image_p.start()
     get_clusters = get_clusters_p.start()
@@ -166,6 +168,8 @@ def start_patch(patch_templates=True):
     heat = heat_p.start()
     heat().stacks.list.side_effect = _get_heat_stack_list
 
+    image_manager = image_manager_p.start()
+
     cinder = cinder_p.start()
     cinder().availability_zones.list.side_effect = _get_availability_zone_list
 
@@ -197,8 +201,8 @@ def start_patch(patch_templates=True):
             return Image('wrong_test')
 
     get_image.side_effect = _get_image
-    nova().images.list_registered.return_value = [Image(),
-                                                  Image(name='wrong_name')]
+    image_manager().list_registered.return_value = [Image(),
+                                                    Image(name='wrong_name')]
     ng_dict = tu.make_ng_dict('ng', '42', ['namenode'], 1)
     cluster = tu.create_cluster('test', 't', 'fake', '0.1', [ng_dict],
                                 id=1, status=c_u.CLUSTER_STATUS_ACTIVE)
@@ -230,7 +234,7 @@ def start_patch(patch_templates=True):
         get_ng_template.side_effect = _get_ng_template
     # request data to validate
     patchers = [get_clusters_p, get_cluster_p,
-                nova_p, get_image_p, heat_p, cinder_p,
+                nova_p, get_image_p, heat_p, image_manager_p, cinder_p,
                 cinder_exists_p]
     if patch_templates:
         patchers.extend([get_ng_template_p, get_ng_templates_p,
@@ -417,7 +421,10 @@ class ValidationTestCase(base.SaharaTestCase):
             'name': 'test-cluster',
             'plugin_name': 'fake',
             'hadoop_version': '0.1',
-            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
+            'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
+            'domain_name': 'domain.org.',
+            'neutron_management_network': 'd9a3bebc-f788-4b81-'
+                                          '9a93-aa048022c1ca'
         }
         self._assert_create_object_validation(data=data)
         data = {
