@@ -13,9 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import uuid
 
-import six
+from oslo_utils import uuidutils
 
 from sahara.utils import files
 
@@ -57,6 +56,7 @@ JOB_TYPE_JAVA = 'Java'
 JOB_TYPE_MAPREDUCE = 'MapReduce'
 JOB_TYPE_SPARK = 'Spark'
 JOB_TYPE_STORM = 'Storm'
+JOB_TYPE_PYLEUS = 'Storm.Pyleus'
 JOB_TYPE_MAPREDUCE_STREAMING = (JOB_TYPE_MAPREDUCE + JOB_TYPE_SEP +
                                 JOB_SUBTYPE_STREAMING)
 JOB_TYPE_PIG = 'Pig'
@@ -71,7 +71,8 @@ JOB_TYPES_ALL = [
     JOB_TYPE_PIG,
     JOB_TYPE_SHELL,
     JOB_TYPE_SPARK,
-    JOB_TYPE_STORM
+    JOB_TYPE_STORM,
+    JOB_TYPE_PYLEUS
 ]
 
 JOB_TYPES_ACCEPTABLE_CONFIGS = {
@@ -82,7 +83,8 @@ JOB_TYPES_ACCEPTABLE_CONFIGS = {
     JOB_TYPE_JAVA: {"configs", "args"},
     JOB_TYPE_SHELL: {"configs", "params", "args"},
     JOB_TYPE_SPARK: {"configs", "args"},
-    JOB_TYPE_STORM: {"args"}
+    JOB_TYPE_STORM: {"args"},
+    JOB_TYPE_PYLEUS: {}
 }
 
 # job actions
@@ -149,7 +151,7 @@ def get_builtin_binaries(job, configs):
     if job.type == JOB_TYPE_JAVA:
         if is_adapt_for_oozie_enabled(configs):
             path = 'service/edp/resources/edp-main-wrapper.jar'
-            name = 'builtin-%s.jar' % six.text_type(uuid.uuid4())
+            name = 'builtin-%s.jar' % uuidutils.generate_uuid()
             return [{'raw': files.get_file_binary(path),
                      'name': name}]
     return []
